@@ -1,7 +1,7 @@
 /* 
    Project: DataBasin
 
-   Copyright (C) 2008-2013 Free Software Foundation
+   Copyright (C) 2008-2014 Free Software Foundation
 
    Author: Riccardo Mottola
 
@@ -25,10 +25,10 @@
 */
 
 #import "AppController.h"
-#import "DBSoap.h"
-#import "DBSoapCSV.h"
-#import "DBCVSWriter.h"
-#import "DBCVSReader.h"
+#import <DataBasinKit/DBSoap.h>
+#import <DataBasinKit/DBSoapCSV.h>
+#import <DataBasinKit/DBCSVWriter.h>
+#import <DataBasinKit/DBCSVReader.h>
 #import "DBLogger.h"
 #import "DBProgress.h"
 #import "Preferences.h"
@@ -123,6 +123,7 @@
 {
   NSUserDefaults *defaults;
   id obj;
+  int size;
   
   defaults = [NSUserDefaults standardUserDefaults];
   
@@ -135,16 +136,20 @@
     }
 
   [logger setLogLevel: [obj intValue]];
-
-  obj = [defaults objectForKey:@"UpBatchSize"];
-  if (obj)
+  
+  size = [defaults integerForKey:@"UpBatchSize"];
+  if (size > 0)
     {
-      int size;
-
-      size = [obj intValue];
-      if (size > 0)
-	[db setUpBatchSize:size];
+      [db setUpBatchSize:size];
     }
+
+  size = [defaults integerForKey:@"DownBatchSize"];
+  if (size > 0)
+    {
+      [db setDownBatchSize:size];
+    }
+
+  // FIXME here we should set the defaults of the CSV reader/writers
 }
 
 - (IBAction)showPrefPanel:(id)sender
@@ -161,7 +166,7 @@
 
   /* Apply defaults */
   defaults = [NSUserDefaults standardUserDefaults];
-  [logger setLogLevel: [[defaults valueForKey: @"StringEncoding"] intValue]];
+  [logger setLogLevel: [defaults integerForKey: @"StringEncoding"]];
 }
 
 
@@ -220,26 +225,14 @@
   NSString *userName;
   NSString *password;
   NSString *token;
-  NSString *urlStr;
   NSURL    *url;
   NSDictionary *uInfo;
-  BOOL useHttps;
-  NSString *protocolString;
   NSMutableDictionary *loginSet;
   
   userName = [fieldUserName stringValue];
   password = [fieldPassword stringValue];
   token = [fieldToken stringValue];
 
-  useHttps = NO;
-  if ([[[NSUserDefaults standardUserDefaults] objectForKey: @"UseHttps"] intValue] == NSOnState)
-    useHttps = YES;
-
-  if (useHttps)
-    protocolString = @"https://";
-  else
-    protocolString = @"http://";
-
   /* if present, we append the security token to the password */
   if (token != nil)
     password = [password stringByAppendingString:token];
@@ -249,17 +242,16 @@
   dbCsv = [[DBSoapCSV alloc] init];
   [dbCsv setDBSoap:db];
   
-  urlStr = nil;
+  url = nil;
   if ([popupEnvironment indexOfSelectedItem] == DB_ENVIRONMENT_PRODUCTION)
-    urlStr = [protocolString stringByAppendingString: @"www.salesforce.com/services/Soap/u/25.0"];
+    url = [DBSoap loginURLProduction];
   else if ([popupEnvironment indexOfSelectedItem] == DB_ENVIRONMENT_SANDBOX)
-    urlStr = [protocolString stringByAppendingString: @"test.salesforce.com/services/Soap/u/25.0"];
+    url = [DBSoap loginURLTest];
 
-  [logger log:LogStandard :@"[AppController doLogin] Url: %@\n", urlStr];  
-  url = [NSURL URLWithString:urlStr];
+  [logger log:LogStandard :@"[AppController doLogin] Url: %@\n", [url absoluteString]];  
   
   NS_DURING
-    [db login :url :userName :password :useHttps];
+    [db login :url :userName :password :YES];
     
     /* session inspector fields */
     [fieldSessionId setStringValue:[db sessionId]];
@@ -271,14 +263,14 @@
     
     /* user inspector fields */
     uInfo = [db userInfo];
-    [fieldOrgName setStringValue: [uInfo valueForKey:@"organizationName"]];
-    [fieldOrgId setStringValue: [uInfo valueForKey:@"organizationId"]];
-    [fieldUserNameInsp setStringValue: [uInfo valueForKey:@"userName"]];
-    [fieldUserFullName setStringValue: [uInfo valueForKey:@"userFullName"]];
-    [fieldUserEmail setStringValue: [uInfo valueForKey:@"userEmail"]];
-    [fieldUserId setStringValue: [uInfo valueForKey:@"userId"]];
-    [fieldProfileId setStringValue: [uInfo valueForKey:@"profileId"]];
-    [fieldRoleId setStringValue: [uInfo valueForKey:@"roleId"]];
+    [fieldOrgName setStringValue: [uInfo objectForKey:@"organizationName"]];
+    [fieldOrgId setStringValue: [uInfo objectForKey:@"organizationId"]];
+    [fieldUserNameInsp setStringValue: [uInfo objectForKey:@"userName"]];
+    [fieldUserFullName setStringValue: [uInfo objectForKey:@"userFullName"]];
+    [fieldUserEmail setStringValue: [uInfo objectForKey:@"userEmail"]];
+    [fieldUserId setStringValue: [uInfo objectForKey:@"userId"]];
+    [fieldProfileId setStringValue: [uInfo objectForKey:@"profileId"]];
+    [fieldRoleId setStringValue: [uInfo objectForKey:@"roleId"]];
 
   NS_HANDLER
     [logger log:LogStandard :@"Login failed\n"];
@@ -375,13 +367,16 @@
 
 - (IBAction)executeSelect:(id)sender
 {
-  NSString      *statement;
-  NSString      *filePath;
-  NSFileHandle  *fileHandle;
-  NSFileManager *fileManager;
-  DBCVSWriter   *cvsWriter;
-  DBProgress    *progress;
+  NSString       *statement;
+  NSString       *filePath;
+  NSFileHandle   *fileHandle;
+  NSFileManager  *fileManager;
+  DBCSVWriter    *csvWriter;
+  DBProgress     *progress;
+  NSString       *str;
+  NSUserDefaults *defaults;
   
+  defaults = [NSUserDefaults standardUserDefaults];
   statement = [fieldQuerySelect string];
   filePath = [fieldFileSelect stringValue];
   
@@ -403,11 +398,18 @@
   [progress setRemainingTimeField: fieldRTSelect];
   [progress setLogger:logger];
   [progress reset];
-  cvsWriter = [[DBCVSWriter alloc] initWithHandle:fileHandle];
-  [cvsWriter setLogger:logger];
-
+  csvWriter = [[DBCSVWriter alloc] initWithHandle:fileHandle];
+  [csvWriter setLogger:logger];
+  [csvWriter setWriteFieldsOrdered:([orderedWritingSelect state] == NSOnState)];
+  str = [defaults stringForKey:@"CSVWriteQualifier"];
+  if (str)
+    [csvWriter setQualifier:str];
+  str = [defaults stringForKey:@"CSVWriteSeparator"];
+  if (str)
+    [csvWriter setSeparator:str];
+  
   NS_DURING
-    [dbCsv query :statement queryAll:([queryAllSelect state] == NSOnState) toWriter:cvsWriter progressMonitor:progress];
+    [dbCsv query :statement queryAll:([queryAllSelect state] == NSOnState) toWriter:csvWriter progressMonitor:progress];
   NS_HANDLER
     if ([[localException name] hasPrefix:@"DB"])
       {
@@ -415,7 +417,7 @@
         [faultPanel makeKeyAndOrderFront:nil];
       }
   NS_ENDHANDLER
-  [cvsWriter release];
+  [csvWriter release];
   [fileHandle closeFile];
   [progress release];
 }
@@ -464,14 +466,15 @@
 {
   NSString       *filePath;
   NSString       *resFilePath;
-  DBCVSReader    *reader;
+  DBCSVReader    *reader;
   NSString       *intoWhichObject;
   DBProgress     *progress;
   NSMutableArray *results;
   NSFileManager  *fileManager;
   NSFileHandle   *resFH;
   NSUserDefaults *defaults;
-  DBCVSWriter    *resWriter;
+  DBCSVWriter    *resWriter;
+  NSString       *str;
   
   defaults = [NSUserDefaults standardUserDefaults];  
   filePath = [fieldFileInsert stringValue];
@@ -491,7 +494,15 @@
   [progress reset];
   
   results = nil;
-  reader = [[DBCVSReader alloc] initWithPath:filePath withLogger:logger];
+  reader = [[DBCSVReader alloc] initWithPath:filePath withLogger:logger];
+  str = [defaults stringForKey:@"CSVReadQualifier"];
+  if (str)
+    [reader setQualifier:str];
+  str = [defaults stringForKey:@"CSVReadSeparator"];
+  if (str)
+    [reader setSeparator:str];
+  [reader parseHeaders];
+
   NS_DURING
     results = [dbCsv create:intoWhichObject fromReader:reader progressMonitor:progress];
     [results retain];
@@ -517,14 +528,28 @@
     }
   else
     {
-      resWriter = [[DBCVSWriter alloc] initWithHandle:resFH];
-      [resWriter setLogger:logger];
-      [resWriter setStringEncoding: [[defaults valueForKey: @"StringEncoding"] intValue]];
-      
-      [resWriter setFieldNames:[results objectAtIndex: 0] andWriteIt:YES];
-      [resWriter writeDataSet: results];
-      
-      [resWriter release];
+      if (results != nil && [results count] > 0)
+        {
+          resWriter = [[DBCSVWriter alloc] initWithHandle:resFH];
+          [resWriter setLogger:logger];
+          [resWriter setStringEncoding: [defaults integerForKey: @"StringEncoding"]];
+          str = [defaults stringForKey:@"CSVWriteQualifier"];
+          if (str)
+            [resWriter setQualifier:str];
+          str = [defaults stringForKey:@"CSVWriteSeparator"];
+          if (str)
+            [resWriter setSeparator:str];
+          
+          
+          [resWriter setFieldNames:[results objectAtIndex: 0] andWriteThem:YES];
+          [resWriter writeDataSet: results];
+          
+          [resWriter release];
+        }
+      else
+        {
+          [logger log:LogStandard :@"[AppController executeInsert] No Results"];
+        }
     }
 
   [reader release];
@@ -577,14 +602,15 @@
 {
   NSString       *filePath;
   NSString       *resFilePath;
-  DBCVSReader    *reader;
+  DBCSVReader    *reader;
   NSString       *whichObject;
   DBProgress     *progress;
   NSMutableArray *results;
   NSFileManager  *fileManager;
   NSFileHandle   *resFH;
   NSUserDefaults *defaults;
-  DBCVSWriter    *resWriter;
+  DBCSVWriter    *resWriter;
+  NSString       *str;
 
   defaults = [NSUserDefaults standardUserDefaults];  
   filePath = [fieldFileUpdate stringValue];
@@ -602,7 +628,15 @@
   [logger log:LogInformative :@"[AppController executeUpdate] object: %@\n", whichObject];
   
   results = nil;
-  reader = [[DBCVSReader alloc] initWithPath:filePath withLogger:logger];
+  reader = [[DBCSVReader alloc] initWithPath:filePath withLogger:logger];
+  str = [defaults stringForKey:@"CSVReadQualifier"];
+  if (str)
+    [reader setQualifier:str];
+  str = [defaults stringForKey:@"CSVReadSeparator"];
+  if (str)
+    [reader setSeparator:str];
+  [reader parseHeaders];
+
   NS_DURING
     results = [dbCsv update:whichObject fromReader:reader progressMonitor:progress];
     [results retain];
@@ -627,14 +661,28 @@
     }
   else
     {
-      resWriter = [[DBCVSWriter alloc] initWithHandle:resFH];
-      [resWriter setLogger:logger];
-      [resWriter setStringEncoding: [[defaults valueForKey: @"StringEncoding"] intValue]];
-      
-      [resWriter setFieldNames:[results objectAtIndex: 0] andWriteIt:YES];
-      [resWriter writeDataSet: results];
-      
-      [resWriter release];
+      if (results != nil && [results count] > 0)
+        {
+          resWriter = [[DBCSVWriter alloc] initWithHandle:resFH];
+          [resWriter setLogger:logger];
+          [resWriter setStringEncoding: [defaults integerForKey: @"StringEncoding"]];
+          str = [defaults stringForKey:@"CSVWriteQualifier"];
+          if (str)
+            [resWriter setQualifier:str];
+          str = [defaults stringForKey:@"CSVWriteSeparator"];
+          if (str)
+            [resWriter setSeparator:str];
+
+          
+          [resWriter setFieldNames:[results objectAtIndex: 0] andWriteThem:YES];
+          [resWriter writeDataSet: results];
+          
+          [resWriter release];
+        }
+      else
+        {
+          [logger log:LogStandard :@"[AppController executeUpdate] No Results"];
+        }
     }
     
   [reader release];
@@ -683,16 +731,20 @@
 
 - (IBAction)executeSelectIdentify:(id)sender
 {
-  NSString      *statement;
-  NSString      *filePathIn;
-  NSString      *filePathOut;
-  NSFileHandle  *fileHandleOut;
-  NSFileManager *fileManager;
-  DBCVSWriter   *cvsWriter;
-  DBCVSReader   *cvsReader;
-  DBProgress    *progress;
-  int           batchSize;
-  
+  NSString       *statement;
+  NSString       *filePathIn;
+  NSString       *filePathOut;
+  NSFileHandle   *fileHandleOut;
+  NSFileManager  *fileManager;
+  DBCSVWriter    *csvWriter;
+  DBCSVReader    *csvReader;
+  DBProgress     *progress;
+  int            batchSize;
+  NSString       *str;
+  NSUserDefaults *defaults;
+
+  defaults = [NSUserDefaults standardUserDefaults];
+
   statement = [fieldQuerySelectIdentify string];
   filePathIn = [fieldFileSelectIdentifyIn stringValue];
   filePathOut = [fieldFileSelectIdentifyOut stringValue];
@@ -719,12 +771,18 @@
   
   fileManager = [NSFileManager defaultManager];
 
-  cvsReader = [[DBCVSReader alloc] initWithPath:filePathIn withLogger:logger];
-
+  csvReader = [[DBCSVReader alloc] initWithPath:filePathIn withLogger:logger];
+  str = [defaults stringForKey:@"CSVReadQualifier"];
+  if (str)
+    [csvReader setQualifier:str];
+  str = [defaults stringForKey:@"CSVReadSeparator"];
+  if (str)
+    [csvReader setSeparator:str];
+  [csvReader parseHeaders];
   if ([fileManager createFileAtPath:filePathOut contents:nil attributes:nil] == NO)
     {
       NSRunAlertPanel(@"Attention", @"Could not create File.", @"Ok", nil, nil);
-      [cvsReader release];
+      [csvReader release];
       return;
     }  
 
@@ -732,12 +790,20 @@
   if (fileHandleOut == nil)
     {
       NSRunAlertPanel(@"Attention", @"Cannot create File.", @"Ok", nil, nil);
-      [cvsReader release];
+      [csvReader release];
       return;
     }
 
-  cvsWriter = [[DBCVSWriter alloc] initWithHandle:fileHandleOut];
-  [cvsWriter setLogger:logger];
+  csvWriter = [[DBCSVWriter alloc] initWithHandle:fileHandleOut];
+  [csvWriter setLogger:logger];
+  [csvWriter setWriteFieldsOrdered:([orderedWritingSelectIdent state] == NSOnState)];
+  str = [defaults stringForKey:@"CSVWriteQualifier"];
+  if (str)
+    [csvWriter setQualifier:str];
+  str = [defaults stringForKey:@"CSVWriteSeparator"];
+  if (str)
+    [csvWriter setSeparator:str];
+
   progress = [[DBProgress alloc] init];
   [progress setLogger:logger];
   [progress setProgressIndicator: progIndSelectIdent];
@@ -745,7 +811,7 @@
   [progress reset];
 
   NS_DURING
-    [dbCsv queryIdentify :statement queryAll:([queryAllSelectIdentify state] == NSOnState) fromReader:cvsReader toWriter:cvsWriter withBatchSize:batchSize progressMonitor:progress];
+    [dbCsv queryIdentify :statement queryAll:([queryAllSelectIdentify state] == NSOnState) fromReader:csvReader toWriter:csvWriter withBatchSize:batchSize progressMonitor:progress];
   NS_HANDLER
     if ([[localException name] hasPrefix:@"DB"])
       {
@@ -754,8 +820,8 @@
       }
   NS_ENDHANDLER
 
-  [cvsReader release];
-  [cvsWriter release];
+  [csvReader release];
+  [csvWriter release];
   [fileHandleOut closeFile];
   
   [progress release];
@@ -792,12 +858,13 @@
 
 - (IBAction)executeDescribe:(id)sender
 {
-  NSString      *filePath;
-  DBCVSWriter   *writer;
-  NSString      *whichObject;
-  NSFileManager *fileManager;
-  NSFileHandle  *fileHandle;
+  NSString       *filePath;
+  DBCSVWriter    *writer;
+  NSString       *whichObject;
+  NSFileManager  *fileManager;
+  NSFileHandle   *fileHandle;
   NSUserDefaults *defaults;
+  NSString       *str;
 
   defaults = [NSUserDefaults standardUserDefaults];
     
@@ -817,9 +884,15 @@
       NSRunAlertPanel(@"Attention", @"Cannot create File.", @"Ok", nil, nil);
     }
   
-  writer = [[DBCVSWriter alloc] initWithHandle:fileHandle];
+  writer = [[DBCSVWriter alloc] initWithHandle:fileHandle];
   [writer setLogger:logger];
-  [writer setStringEncoding: [[defaults valueForKey: @"StringEncoding"] intValue]];
+  [writer setStringEncoding: [defaults integerForKey: @"StringEncoding"]];
+  str = [defaults stringForKey:@"CSVWriteQualifier"];
+  if (str)
+    [writer setQualifier:str];
+  str = [defaults stringForKey:@"CSVWriteSeparator"];
+  if (str)
+    [writer setSeparator:str];
   
   whichObject = [[[popupObjectsDescribe selectedItem] title] retain];
   NSLog(@"object: %@", whichObject);
@@ -930,13 +1003,14 @@
 {
   NSString       *filePath;
   NSString       *resFilePath;
-  DBCVSReader    *reader;
-  DBCVSWriter    *resWriter;
+  DBCSVReader    *reader;
+  DBCSVWriter    *resWriter;
   NSMutableArray *results;
   NSFileManager  *fileManager;
   NSFileHandle   *resFH;
   NSUserDefaults *defaults;
   DBProgress     *progress;
+  NSString       *str;
 
   defaults = [NSUserDefaults standardUserDefaults];  
   filePath = [fieldFileDelete stringValue];
@@ -944,7 +1018,14 @@
 
   NSLog(@"writing results to: %@", resFilePath);
     
-  reader = [[DBCVSReader alloc] initWithPath:filePath byParsingHeaders:([checkSkipFirstLine state]==NSOnState) withLogger:logger];
+  reader = [[DBCSVReader alloc] initWithPath:filePath byParsingHeaders:([checkSkipFirstLine state]==NSOnState) withLogger:logger];
+  str = [defaults stringForKey:@"CSVReadQualifier"];
+  if (str)
+    [reader setQualifier:str];
+  str = [defaults stringForKey:@"CSVReadSeparator"];
+  if (str)
+    [reader setSeparator:str];
+  /* no need to reparse the headers since they are not used, just skipped */
 
   progress = [[DBProgress alloc] init];
   [progress setProgressIndicator: progIndDelete];
@@ -977,11 +1058,17 @@
     }
   else
     {
-      resWriter = [[DBCVSWriter alloc] initWithHandle:resFH];
+      resWriter = [[DBCSVWriter alloc] initWithHandle:resFH];
       [resWriter setLogger:logger];
-      [resWriter setStringEncoding: [[defaults valueForKey: @"StringEncoding"] intValue]];
+      [resWriter setStringEncoding: [defaults integerForKey: @"StringEncoding"]];
+      str = [defaults stringForKey:@"CSVWriteQualifier"];
+      if (str)
+        [resWriter setQualifier:str];
+      str = [defaults stringForKey:@"CSVWriteSeparator"];
+      if (str)
+        [resWriter setSeparator:str];
       
-      [resWriter setFieldNames:[results objectAtIndex: 0] andWriteIt:YES];
+      [resWriter setFieldNames:[results objectAtIndex: 0] andWriteThem:YES];
       [resWriter writeDataSet: results];
       
       [resWriter release];
