@@ -1,7 +1,7 @@
 /*
  Project: DataBasin
  
- Copyright (C) 2010-2013 Free Software Foundation
+ Copyright (C) 2010-2014 Free Software Foundation
  
  Author: Riccardo Mottola
  
@@ -24,8 +24,9 @@
 
 
 #import "DBObjectInspector.h"
-#import "DBSObject.h"
+#import <DataBasinKit/DBSObject.h>
 
+#import "DBTextFormatter.h"
 
 @implementation DBObjectInspector
 
@@ -53,6 +54,12 @@
 {
   NSTableColumn *col;
   NSCell *cell;
+  DBTextFormatter *tf;
+  
+  tf = [[DBTextFormatter alloc] init];
+  [tf setMaxLength:18];
+  [fieldObjId setFormatter:tf];
+  [tf release];
 
   col = [fieldTable tableColumnWithIdentifier:COLID_LABEL];
   cell = [col dataCell];
